@@ -1,11 +1,11 @@
-# $Id: TCP.pm 1957 2006-05-15 03:17:21Z immute $
+# $Id: TCP.pm 1994 2006-06-17 22:40:25Z rcaputo $
 
 package POE::Component::Client::TCP;
 
 use strict;
 
 use vars qw($VERSION);
-$VERSION = do {my($r)=(q$Revision: 1957 $=~/(\d+)/);sprintf"1.%04d",$r};
+$VERSION = do {my($r)=(q$Revision: 1994 $=~/(\d+)/);sprintf"1.%04d",$r};
 
 use Carp qw(carp croak);
 use Errno qw(ETIMEDOUT ECONNRESET);
@@ -57,6 +57,7 @@ sub new {
   foreach (
     qw( Connected ConnectError Disconnected ServerInput
       ServerError ServerFlushed Started
+      ServerHigh ServerLow
     )
   ) {
     croak "$_ must be a coderef" if(
@@ -64,6 +65,20 @@ sub new {
     );
   }
 
+  my $high_mark_level = delete $param{HighMark};
+  my $low_mark_level  = delete $param{LowMark};
+  my $high_event      = delete $param{ServerHigh};
+  my $low_event       = delete $param{ServerLow};
+
+  # this is ugly, but now its elegant :)  grep++
+  my $foo = grep { defined $_ } ($high_mark_level, $low_mark_level, $high_event, $low_event);
+  if ($foo > 0) {
+    croak "If you use the Mark settings, you must define all four" unless $foo == 4;
+  }
+
+  $high_event = sub { } unless defined $high_event;
+  $low_event  = sub { } unless defined $low_event;
+
   my $conn_callback       = delete $param{Connected};
   my $conn_error_callback = delete $param{ConnectError};
   my $disc_callback       = delete $param{Disconnected};
@@ -125,7 +140,7 @@ sub new {
   # Spawn the session that makes the connection and then interacts
   # with what was connected to.
 
-  $session_type->create
+  return $session_type->create
     ( @$session_params,
       inline_states =>
       { _start => sub {
@@ -189,11 +204,21 @@ sub new {
               InputEvent   => 'got_server_input',
               ErrorEvent   => 'got_server_error',
               FlushedEvent => 'got_server_flush',
+              do {
+                  $foo ? return (
+                    HighMark => $high_mark_level,
+                    HighEvent => 'got_high',
+                    LowMark => $low_mark_level,
+                    LowEvent => 'got_low',
+                  ) : ();
+                },
             );
 
           $heap->{connected} = 1;
           $conn_callback->(@_);
         },
+        got_high => $high_event,
+        got_low => $low_event,
 
         got_connect_error => sub {
           my $heap = $_[HEAP];
@@ -273,7 +298,7 @@ sub new {
       # User supplied states.
       package_states => $package_states,
       object_states  => $object_states,
-    );
+    )->ID;
 }
 
 sub _get_filter {
@@ -336,7 +361,7 @@ POE::Component::Client::TCP - a simplified TCP client
 
   # Complete usage.
 
-  POE::Component::Client::TCP->new
+  my $session_id = POE::Component::Client::TCP->new
     ( RemoteAddress  => "127.0.0.1",
       RemotePort     => "chargen",
       BindAddress    => "127.0.0.1",
@@ -425,6 +450,10 @@ with as little work as possible.
 
 =head1 Constructor Parameters
 
+The new() method can accept quite a lot of parameters.  It will return
+the session ID of the accecptor session.  One must use callbacks to 
+check for errors rather than the return value of new().
+
 =over 2
 
 =item Alias
@@ -661,16 +690,18 @@ POE::Wheel::ReadWrite, POE::Filter
 
 =head1 CAVEATS
 
-This may not be suitable for complex client tasks.
+This may not be suitable for complex client tasks.  After a point, it
+becomes easier to roll a custom client using POE::Wheel::SocketFactory
+and POE::Wheel::ReadWrite.
 
 This looks nothing like what Ann envisioned.
 
 =head1 AUTHORS & COPYRIGHTS
 
-POE::Component::Client::TCP is Copyright 2001 by Rocco Caputo.  All
-rights are reserved.  POE::Component::Client::TCP is free software,
-and it may be redistributed and/or modified under the same terms as
-Perl itself.
+POE::Component::Client::TCP is Copyright 2001-2006 by Rocco Caputo.
+All rights are reserved.  POE::Component::Client::TCP is free
+software, and it may be redistributed and/or modified under the same
+terms as Perl itself.
 
 POE::Component::Client::TCP is based on code, used with permission,
 from Ann Barcomb E<lt>kudra@domaintje.comE<gt>.
