@@ -1,4 +1,4 @@
-# $Id: Event.pm 1903 2006-03-20 04:44:08Z rcaputo $
+# $Id: Event.pm 2017 2006-08-03 16:05:34Z rcaputo $
 
 # Event.pm event loop bridge for POE::Kernel.
 
@@ -8,7 +8,7 @@ package POE::Loop::Event;
 use strict;
 
 use vars qw($VERSION);
-$VERSION = do {my($r)=(q$Revision: 1903 $=~/(\d+)/);sprintf"1.%04d",$r};
+$VERSION = do {my($r)=(q$Revision: 2017 $=~/(\d+)/);sprintf"1.%04d",$r};
 
 # Everything plugs into POE::Kernel.
 package POE::Kernel;
@@ -36,11 +36,12 @@ sub loop_initialize {
   $_watcher_timer = Event->timer(
     cb     => \&_loop_event_callback,
     after  => 0,
-    parked => 1,
   );
 }
 
 sub loop_finalize {
+  my $self = shift;
+
   foreach my $fd (0..$#fileno_watcher) {
     next unless defined $fileno_watcher[$fd];
     foreach my $mode (MODE_RD, MODE_WR, MODE_EX) {
@@ -49,6 +50,10 @@ sub loop_finalize {
       ) if defined $fileno_watcher[$fd]->[$mode];
     }
   }
+
+  foreach my $signal (keys %signal_watcher) {
+    $self->loop_ignore_signal($signal);
+  }
 }
 
 #------------------------------------------------------------------------------
